package risc.gui;

/**
 * Display of the registers
 *
 * 30.04.01 by Benjamin Leroy-Beaulieu
 *
 */

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.table.TableColumn;

import risc.RISC;

public class RegistersGUI {

    public JFrame       frame;
    private Object[][]     registersValuesTable = new Object[RISC.register_count + 2][2];  // !!! + 2
    private String[]       columnNames = new String[2];
    private RiscTable      registersTable;
    private JTable         displayJTable;
    private JScrollPane    scrollPane;

    /** pointer to the main startGUI */
    private final StartGUI startGUI;

    public RegistersGUI(StartGUI thisStartGUI) {
	
	startGUI = thisStartGUI;
	
	/** Creating the frame */
	Toolkit tk  = Toolkit.getDefaultToolkit();
	Dimension d = tk.getScreenSize();
	int screenw = d.width;
	int screenh = d.height;
	frame = new JFrame("Registers");
	frame.setSize(2*screenw/10,2*screenh/3);
	frame.setLocation(3*screenw/10,screenh/3);

	/** Allowing window closing */
        frame.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);

	/** Layout Manager */
	frame.getContentPane().setLayout(new GridBagLayout());

	/** Constraint object */  
        GridBagConstraints constraints = new GridBagConstraints();
	// components grow only horitontaly 
        constraints.fill    = GridBagConstraints.BOTH; 
	// 5-pixel margin on all sides
        constraints.insets  = new Insets(5,5,5,5); 
        constraints.weightx = 1.0;
        constraints.weighty = 1.0;   

	// filling the registers
	for (int i=0; i<RISC.register_count; i++) {
	    //  registers[i] = i;
	    registersValuesTable[i][0] = startGUI.writer.getRegisterName(i);
	    registersValuesTable[i][1] = new Integer(startGUI.getEmulator().getRx(i));
	} // for
        registersValuesTable[RISC.register_count][0] = "PC";
        registersValuesTable[RISC.register_count][1] = new Integer(startGUI.getEmulator().getPC());
        registersValuesTable[RISC.register_count+1][0] = "IR";
        registersValuesTable[RISC.register_count+1][1] = startGUI.writer.getInstructionString(startGUI.getEmulator().getIR());

	/** preparing the columnNames */
	columnNames[0] = "Reg";
	columnNames[1] = "Content";

	/** filling the registerTable */
	registersTable = new RiscTable(columnNames, registersValuesTable);

	/** turn the registerTable into a JTable */
	displayJTable = new JTable(registersTable);

	/** Setting the preferred size for the columns */
	TableColumn column = null;
	column = displayJTable.getColumnModel().getColumn(0);
	column.setPreferredWidth(30);
	column.setMaxWidth(40);

	/** putting the JTable in a scrollPane */
	scrollPane = new JScrollPane(displayJTable);
	displayJTable.setPreferredScrollableViewportSize(new Dimension(80, 300));

	/** Putting the scrollPane in the main frame */
	constraints.gridx = 0;
	constraints.gridy = 0;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
	frame.getContentPane().add(scrollPane, constraints);

	// !!! frame.setVisible(true);

    } // Registers contructor

    protected void updateData() {
	for (int i=0; i<RISC.register_count; i++) {
	    registersTable.setValueAt(new Integer(startGUI.getEmulator().getRx(i)), i, 1);
	} // for
        registersTable.setValueAt(new Integer(startGUI.getEmulator().getPC()), RISC.register_count, 1); // !!!
        registersTable.setValueAt(startGUI.writer.getInstructionString(startGUI.getEmulator().getIR()), RISC.register_count+1, 1); // !!!
    }
} // Registers class
