package risc.gui;

import risc.RISC;
import risc.emulator.Emulator;

public class RiscDebugger {

    //########################################################################
    // RiscDebugger interface

    public static String usage() {
        return
            "usage: java " +RiscDebugger.class.getName()+ " [options] [filename]\n" +
            "\n" +
            "where options include:\n" +
            "    -mem=size  set the machine memory size to \"size\"\n" +
            "    -verbose   enable verbose output\n" +
            "    -? -help   print this help message and exit\n";
    }

    public static int run(String[] args) {
        String filename = null;
        boolean verbose = false;
        boolean help = false;
        int size = Emulator.DEFAULT_MEMORY_SIZE;

        for (int i = 0; i < args.length; i++) {
            if (args[i].equals("-?") || args[i].equals("-help")) {
                help = true;
            } else if (args[i].equals("-verbose")) {
                verbose = true;
            } else if (args[i].startsWith("-mem=")) {
                String string = args[i].substring(5);
                try {
                    int value = Integer.parseInt(string);
                    if (value < 0) {
                        System.err.println("invalid memory size: " + value);
                        return 1;
                    } else {
                        size = value;
                    }
                } catch (NumberFormatException exception) {
                    System.err.println("invalid memory size: " + string);
                    return 1;
                }
            } else if (args[i].startsWith("-")) {
                System.err.println("unrecognized option: " + args[i]);
                System.err.print(usage());
                return 1;
            } else if (filename == null) {
                filename = args[i];
            } else {
                System.err.println("wrong argument: " + args[i]);
                System.err.print(usage());
                return 1;
            }
        }

        if (help) {
            System.out.print(usage());
            return 0;
        }

	StartGUI start = new StartGUI(size, verbose, filename);

        return 0;
    }

    public static void main(String[] args) {
        int exitcode = run(args);
        if (exitcode != 0) System.exit(exitcode);
    }
    
    //########################################################################
}
