package risc.gui;

/**
 * A table model for the risc debugger
 *
 * 01.05.01 by Benjamin Leroy-Beaulieu
 *
 */

import javax.swing.table.AbstractTableModel;
import javax.swing.JOptionPane;

public class RiscTable extends AbstractTableModel {
    
    private String[] columnNames;
    private Object[][] data;

    public RiscTable(String[] columnNames, Object[][] data)
	throws ArrayIndexOutOfBoundsException {

	this.columnNames = new String[columnNames.length];
	for (int col=0; col<columnNames.length; col++) 
	    this.columnNames[col] = columnNames[col];

	this.data = new Object[data.length][columnNames.length];
	for (int row=0; row<data.length; row++) 
	    for (int col=0; col<columnNames.length; col++)
		this.data[row][col] = data[row][col];

    }

    public int getColumnCount() {
	return columnNames.length;
    }

    public int getRowCount() {
	return data.length;
    }

    public String getColumnName(int col) {
	return columnNames[col];
    }

    public Object getValueAt(int row, int col) {
	return data[row][col];
    }
    
    /**
     * This method is for JTable do determine the defauld renderer/editor
     * for each cell. If we didn't implement this method, then a boolean
     * column would contain text ("true"/"false") rather than a check box.
     */
    // !!! public Class getColumnClass(int col) {
    // !!! return getValueAt(0, col).getClass();
    // !!!}

    /** 
     * Since we do not want our cells to be editable, we do not implement
     * the public boolean isCellEditable(int row, int col) method.
     */
    
    /**
     * The following method is implemented since the table's data can change
     */
    public void setValueAt(Object value, int row, int col) {
	
	// Debug code not necessary here
        /* !!!
	if (data[0][col] instanceof Integer && !(value instanceof Integer)) {
	    try {
		data[row][col] = new Integer(value.toString());
		fireTableCellUpdated(row, col);
	    }
	    catch(NumberFormatException e) {
		JOptionPane.showMessageDialog(null, 
					      "The \"" + getColumnName(col) +
					      "\" column accepts only integer values."
					      );
	    }
	} else {
        */
	    data[row][col] = value;
	    fireTableCellUpdated(row, col);
            // !!! }

    }

    /**
     * allows to change the value, add or delete rows
     */
    public void update(String[] columnNames, Object[][] data) {	
	int oldRowCount = getRowCount();

	this.columnNames = new String[columnNames.length];
	for (int col=0; col<columnNames.length; col++) 
	    this.columnNames[col] = columnNames[col];

	this.data = new Object[data.length][columnNames.length];
	for (int row=0; row<data.length; row++) 
	    for (int col=0; col<columnNames.length; col++)
		this.data[row][col] = data[row][col];

	int newRowCount = getRowCount();

	if (oldRowCount < newRowCount) {
	    fireTableRowsInserted(oldRowCount, newRowCount-1);
	} else if (newRowCount < oldRowCount) {
	    fireTableRowsDeleted(newRowCount, oldRowCount-1);
	}
	for (int row=0; row<newRowCount; row++) 
	    for (int col=0; col<getColumnCount(); col++) 
		fireTableCellUpdated(row, col);

	
    } // update

    public void printDebugData() {
	int numRows = getRowCount();
	int numCols = getColumnCount();

	for (int row=0; row<numRows; row++) {
	    System.out.print("    row " + row + ":\t");
	    for (int col=0; col<numCols; col++)
		System.out.print("" + data[row][col] +"\t");
	    System.out.println();
	}
	System.out.println("-------------------------------------");
    }

} // class RiscTable
