package risc.gui;

/** 
 * Display of the stack content
 *
 * 03.05.01 by Benjamin Leroy-Beaulieu
 *
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.table.*;
import java.util.Vector;
import java.util.zip.DataFormatException;

// !!!
import javax.swing.table.AbstractTableModel;

import risc.StringHelper;
import risc.debugger.Debugger;

import risc.RISC;

public class StackGUI {

    public JFrame         frame;

    private JScrollPane    stackScrollPane;
    // !!! private Object[][]     stackTable;
    // !!! private String[]       columnNames;
    // !!! private RiscTable      stackRiscTable;
    private JTable         stackJTable;
    private DefaultTableCellRenderer fpRenderer;
    private DefaultTableCellRenderer spRenderer;
    private DefaultTableCellRenderer pRenderer;
    private DefaultTableCellRenderer normalRenderer;

    private boolean        hexa;
    private JPanel         hexaPanel;
    private JRadioButton   hexaRadioButton;
    private JRadioButton   deciRadioButton;
    private ButtonGroup    hexaButtonGroup;

    /** Pointer to the main gui */
    private final StartGUI startGUI;

    /** utils for the table up-to-date keeping */
    // !!! private int            instructionNumber;
    // !!! private String         fromRegister, toRegister;
    // !!! private int            stackTableSize;
   
    public StackGUI(StartGUI thisStartGUI) {

	startGUI = thisStartGUI;
	// !!! fromRegister = "4092";
	// !!! toRegister = "SP";
	hexa = false;
	
	/** Creating the frame */
	Toolkit tk  = Toolkit.getDefaultToolkit();
	Dimension d = tk.getScreenSize();
	int screenw = d.width;
	int screenh = d.height;
	frame = new JFrame("Stack");
	frame.setSize(2*screenw/10, screenh);
	frame.setLocation(5*screenw/10,0);

	/** Allowing window closing */
        frame.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
	
	/** frame's layout Manager */
	frame.getContentPane().setLayout(new GridBagLayout());

	/** frame's constraint object */ 
        GridBagConstraints constraints = new GridBagConstraints();
	// components grow in both directions
        constraints.fill    = GridBagConstraints.BOTH; 
	// 5-pixel margin on all sides
        constraints.insets  = new Insets(5,5,5,5); 
        constraints.weightx = 1.0;
        constraints.weighty = 1.0;

	/** Upper pane: stackScrollPane */
	
	/** filling the stackTable */
	// !!! stackTableSize = (from() - to())+1;
	// !!! stackTable = new Object[stackTableSize][2];
	// !!! int j = 0;
	// !!! for (int i=from(); i >= to(); i--) {
	    // Now take care, 'cause the address should be in inversed order
	    // !!! stackTable[j][0] = new Integer(i*4);
	    // !!! stackTable[j][1] = new Integer(startGUI.getEmulator().getWordAt(i*4));
	    // !!! j++;
	// !!! }

	/** preparing the columnNames */
	// !!! columnNames = new String[2];
	// !!! columnNames[0] = "Address";
	// !!! columnNames[1] = "Value";
	
	/** filling the stackRiscTable */
	// !!! stackRiscTable = new RiscTable(columnNames, stackTable);
        model = new MyTableModel();

	/** Constructing three renderers */
	fpRenderer = new DefaultTableCellRenderer();
	fpRenderer.setBackground(Color.green);
	fpRenderer.setHorizontalAlignment(SwingConstants.RIGHT);
	spRenderer = new DefaultTableCellRenderer();
	spRenderer.setBackground(Color.pink);
	spRenderer.setHorizontalAlignment(SwingConstants.RIGHT);
	pRenderer = new DefaultTableCellRenderer();
	pRenderer.setBackground(Color.cyan);
	pRenderer.setHorizontalAlignment(SwingConstants.RIGHT);
	normalRenderer = new DefaultTableCellRenderer();
	normalRenderer.setBackground(Color.white);
	normalRenderer.setHorizontalAlignment(SwingConstants.RIGHT);
	
	/** turn the RiscTable into a JTable */
	stackJTable = new JTable(model) {
            public TableCellRenderer getCellRenderer(int row, int column) {
                int adr = model.getRowAddress(row);
                if (adr == data().getRx(startGUI.SP)) {
                    if (adr == data().getRx(startGUI.FP)) {
                        return pRenderer;
                    } else {
                        return spRenderer;
                    }
                } else if (adr == data().getRx(startGUI.FP)) {
		    return fpRenderer;
		} else {
		    return normalRenderer;
		}
	    }
	};
        stackJTable.setFont(new Font("Monospaced", Font.PLAIN, stackJTable.getFont().getSize()));

	/** Setting the preferred size for the columns */
	// !!! TableColumn column = null;
	// !!! column = stackJTable.getColumnModel().getColumn(0);
	// !!! column.setPreferredWidth(60);
	// !!! column.setMaxWidth(80);

	/** Creating the ScrollPane */
	stackScrollPane = new JScrollPane(stackJTable);
	stackJTable.setPreferredScrollableViewportSize(new Dimension(80, 500));

	/** puttin the scrollPane in the mainFrame */
	constraints.gridx = 0;
	constraints.gridy = 0;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
        constraints.weightx = 1.0;
        constraints.weighty = 1.0;
	frame.getContentPane().add(stackScrollPane, constraints);
	
	/** 
	 * The radio buttons: hexa / deci
	 */
	hexaPanel = new JPanel();

	/** boundsPanel's layout manager */
	hexaPanel.setLayout(new GridBagLayout());
	constraints.gridx = 0;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
        constraints.weighty = 1.0;
        constraints.weightx = 1.0;

	hexaRadioButton = new JRadioButton("hexadecimal display");
	constraints.gridy = 0;
	hexaPanel.add(hexaRadioButton, constraints);

	deciRadioButton = new JRadioButton("decimal display", true);
	constraints.gridy = 1;
	hexaPanel.add(deciRadioButton, constraints);

	hexaButtonGroup = new ButtonGroup();
	hexaButtonGroup.add(hexaRadioButton);
	hexaButtonGroup.add(deciRadioButton);
	
	constraints.gridx = 0;
	constraints.gridy = 2;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
        constraints.weighty = 0.0;
        constraints.weightx = 0.0;
	frame.getContentPane().add(hexaPanel, constraints);


	hexaRadioButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		if (hexaRadioButton.isSelected()) {
		    hexa = true;
		} else
		    hexa = false;
		update();
	    } // action performed
	}); 

	deciRadioButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		if (deciRadioButton.isSelected()) {
		    hexa = false;
		} else
		    hexa = true;
		update();
	    } // action performed
	}); 

	// !!! frame.setVisible(true);

    } // Stack constructor

    //########################################################################

    public void update() {
        model.fireTableRowsUpdated(0, model.getRowCount() - 1);
    }

    private final MyTableModel model;

    private Debugger data() {
        return startGUI.getEmulator();
    }

    private class MyTableModel extends AbstractTableModel {

        public int getRowAddress(int row) {
            return 4 * (data().getMemorySize() / 4 - row - 1);
        }

        public int getColumnCount() {
            return 2;
        }

        public String getColumnName(int col) {
            switch (col) {
            case 0: return "Address";
            case 1: return "Value";
            default: throw new Error("col = " + col);
            }
        }

        public int getRowCount() {
            // !!! may change
            return data().getMemorySize() / 4;
        }

        public Object getValueAt(int row, int col) {
            int adr = getRowAddress(row);
            switch (col) {
            case 0: return "" + adr;
                // !!! hexa
            case 1: return hexa ?
                        "0x" + StringHelper.toHexString(data().getWordAt(adr),8):
                            StringHelper.toDecString(data().getWordAt(adr));
            default: throw new Error("col = " + col);
            }
        }

    }

} // stack class


// !!!
    //########################################################################

    /*
    private JPanel         boundsPanel;
    private JLabel         boundsLabel;
    private Vector         comboVector;
    private JLabel         fromLabel;
    private JTextField     fromTextField;
    private JComboBox      fromComboBox;
    private JLabel         toLabel;
    private JTextField     toTextField;
    private JComboBox      toComboBox;
    private JButton        displayButton;
    */
	
	/** Constructing the boundsPanel */
/*
	boundsPanel = new JPanel();
*/
	
	/** boundsPanel's layout manager */
/*
	boundsPanel.setLayout(new GridBagLayout());
*/

	/** boundsPanel's constraint object */
/*
        constraints.fill   = GridBagConstraints.BOTH;
        constraints.weightx = 1.0;
        constraints.weighty = 1.0;
*/

	/** boundsLabel */
/*
	boundsLabel = new JLabel("Display Stack:");
	constraints.gridx = 0;
	constraints.gridy = 0;
	constraints.gridwidth = 3;
	constraints.gridheight = 1;
        constraints.weightx = 0.0;
        constraints.fill   = GridBagConstraints.NONE;
	boundsPanel.add(boundsLabel, constraints);
*/

	/** Building the vector for the combo boxes */
/*
	comboVector = new Vector(33,1);
	comboVector.addElement("<-");
	comboVector.addElement("4092");
	for(int i=RISC.register_count - 1; i>-1; i--) {
	    comboVector.addElement(startGUI.writer.getRegisterName(i));
	} // for
*/

	/** fromLabel */
/*
	fromLabel = new JLabel("from");
	constraints.gridx = 0;
	constraints.gridy = 1;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
        constraints.weightx = 0.0;
        constraints.fill   = GridBagConstraints.BOTH;
	boundsPanel.add(fromLabel, constraints);
*/
	
	/** fromTextField*/
/*
	fromTextField = new JTextField(40);
	constraints.gridx = 1;
	constraints.gridy = 1;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
        constraints.weightx = 1.0;
	boundsPanel.add(fromTextField, constraints);
	fromTextField.setText("");
*/

	/** fromComboBox */
/*
	fromComboBox = new JComboBox(comboVector);
	constraints.gridx = 2;
	constraints.gridy = 1;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
        constraints.weightx = 0.0;
	boundsPanel.add(fromComboBox, constraints);
	fromComboBox.setSelectedItem(fromRegister);
*/
	
	/** toLabel */
/*
	toLabel = new JLabel("to");
	constraints.gridx = 0;
	constraints.gridy = 2;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
        constraints.weightx = 0.0;
	boundsPanel.add(toLabel, constraints);
*/
	
	/** toTextField*/
/*
	toTextField = new JTextField(40);
	constraints.gridx = 1;
	constraints.gridy = 2;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
        constraints.weightx = 1.0;
	boundsPanel.add(toTextField, constraints);
	toTextField.setText("");
*/

	/** toComboBox */
/*
	toComboBox = new JComboBox(comboVector);
	constraints.gridx = 2;
	constraints.gridy = 2;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
        constraints.weightx = 0.0;
	boundsPanel.add(toComboBox, constraints);
	toComboBox.setSelectedItem(toRegister);
*/
	
	/** displayButton */
/*
	displayButton = new JButton("Display");
	constraints.gridx = 0;
	constraints.gridy = 3;
	constraints.gridwidth = 3;
	constraints.gridheight = 1;
        constraints.fill   = GridBagConstraints.NONE;
	boundsPanel.add(displayButton, constraints);
*/
	
	/** Now, add the boundsPanel to the main frame */
/*
	constraints.gridx = 0;
	constraints.gridy = 1;
	constraints.gridwidth = 1;
	constraints.gridheight = 1;
        constraints.weighty = 0.0;
        constraints.weightx = 1.0;
        constraints.fill   = GridBagConstraints.BOTH;
	frame.getContentPane().add(boundsPanel, constraints);
*/


	/***************************************************************
	 *                                                             *
	 *       Now, the action/item listeners                        *
	 *                                                             *
	 ***************************************************************/

	/** fromComboBox */
/*
	fromComboBox.addItemListener(new ItemListener() {
	    public void itemStateChanged(ItemEvent e) {
		fromTextField.setText("");
	    } // actionPerformed
	});
*/

	/** toComboBox */
/*
	toComboBox.addItemListener(new ItemListener() {
	    public void itemStateChanged(ItemEvent e) {
		toTextField.setText("");
	    } // actionPerformed
	});
*/

	/** Displaybutton */
/*
	displayButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
		// System.out.println("Somebody clicked on display!");
		String fromString;
		String toString;
		int from, to, oldFrom, oldTo;
		oldFrom = from();
		oldTo = to();
		// System.out.println("Get the new to and from values");
		if (fromTextField.getText().equals("")) {
		    fromString = (String)fromComboBox.getSelectedItem();
		} else {
		    fromString = fromTextField.getText();
		}
		if (toTextField.getText().equals("")) {
		    toString = (String)toComboBox.getSelectedItem();
		} else {
		    toString = toTextField.getText();
		}
		// System.out.println("oldFrom=" + fromRegister);
		// System.out.println("oldTo=  " + toRegister);
		// System.out.println("newFrom=" + fromString);
		// System.out.println("newto=  " + toString);
		// System.out.println();
		// System.out.print("has from changed?  ");
		if (fromString.equalsIgnoreCase(fromRegister)) {
		    // System.out.println("no");
		    // System.out.print("has to changed?  ");
		    if (toString.equalsIgnoreCase(toRegister)) {
			// System.out.println("no");
			// System.out.println("do nothing");
		    } else {
			// System.out.println("yes");
			// System.out.print("Is to a comboBox?  ");
			if (toTextField.getText().equals("")) {
			    // System.out.println("yes");
			    toRegister = toString;
			} else {
			    // System.out.println("no");
			    try {
				// System.out.print("is to an integer?  ");
				to = (new Integer(toString)).intValue();
				// System.out.println("yes");
				try {
				    // System.out.print("is from a register?  ");
				    from = (new Integer(fromRegister)).intValue();
				    // System.out.println("no");
				    // System.out.print("is 0 < to < from?  ");
				    if (to < 0 || to> oldFrom*4) {
					// System.out.println("no");
					popupError("to must be between 0 and from");
				    } else {
					// System.out.println("yes");
					toRegister = toString;
				    } // 0 < to < from
				} catch (NumberFormatException nfe) {
				    // System.out.println("yes");
				    // System.out.print("is 0 < to < 4092?  ");
				    if (to < 0 || to> 4092) {
					// System.out.println("no");
					popupError("to must be between 0 and 4092");
				    } else {
					// System.out.println("yes");
					toRegister = toString;
				    } // 0 < to < from
				} // is from a register
			    } catch (NumberFormatException nfe) {
				// System.out.println("no");
				popupError("to must be an integer");
			    } // is to an integer
			} // is to a comboBox?
		    } // has to changed?
		} else {
		    // System.out.println("yes");
		    // System.out.print("Is from a comboBox?  ");
		    if (fromTextField.getText().equals("")) {
			// System.out.println("yes");
			// System.out.print("Has to changed?  ");
			if (toString.equalsIgnoreCase(toRegister)) {
			    // System.out.println("no");
			    fromRegister = fromString;
			} else {
			    // System.out.println("yes");
			    // System.out.print("Is to a comboBox?  ");
			    if (toTextField.getText().equals("")) {
				// System.out.println("yes");
				fromRegister = fromString;
				toRegister = toString;
			    } else {
				// System.out.println("no");
				try {
				    // System.out.print("Is to an integer?  ");
				    to = (new Integer(toString)).intValue();
				    // System.out.println("yes");
				    // System.out.print("Is to between 0 and 4092?  ");
				    if (to < 0 || to > 4092) {
					// System.out.println("no");
					popupError("to must be between 0 and 4092!");
				    } else {
					// System.out.println("yes");
					fromRegister = fromString;
					toRegister = toString;
				    } // 0 < to < 4092
				} catch (NumberFormatException nfe) {
				    // System.out.println("no");
				    popupError("To must be an integer");
				}// is to an integer?
			    } // is to a combox
			} // had to changed?
		    } else {
			// System.out.println("no");
			try {
			    // System.out.print("Is from an integer?  ");
			    from = (new Integer(fromString)).intValue();
			    // System.out.println("yes");
			    // System.out.print("Has to changed?  ");
			    if (toString.equalsIgnoreCase(toRegister)) {
				// System.out.println("no");
				try {
				    // System.out.print("Is to a register?  ");
				    to = (new Integer(toRegister)).intValue();
				    // System.out.println("no");
				    // System.out.print("Is from between to and 4092?  ");
				    if (from < oldTo*4 || from > 4092) {
					// System.out.println("no");
					popupError("From should be between to and 4092");
				    } else {
					// System.out.println("yes");
					fromRegister = fromString;
				    } // to < from < 4092
				} catch (NumberFormatException nfe) {
				    // System.out.println("yes");
				    // System.out.print("Is from between 0 and 4092?  ");
				    if (from < 0 || from > 4092) {
					// System.out.println("no");
					popupError("From should be between 0 and 4092");
				    } else {
					// System.out.println("yes");
					fromRegister = fromString;
				    } // 0 < from < 4092
				} // is to a register?
			    } else {
				// System.out.println("yes");
				// System.out.print("Is to a combox?  ");
				if (toTextField.getText().equals("")) {
				    // System.out.println("yes");
				    // System.out.print("Is from between 0 and 4092?  ");
				    if (from < 0 || from > 4092) {
					// System.out.println("no");
					popupError("From should be between 0 and 4092");
				    } else {
					// System.out.println("yes");
					fromRegister = fromString;
				    } // 0 < from < 4092
				} else {
				    // System.out.println("no");
				    try {
					// System.out.print("Is to an integer?  ");
					to = (new Integer(toString)).intValue();
					// System.out.println("yes");
					// System.out.print("Are to & from compatible?  ");
					if (0 > to || to > from || from > 4092) {
					    // System.out.println("no");
					    popupError("\"0 <= to <= from <= 4092\" must hold!");
					} else {
					    // System.out.println("yes");
					    fromRegister = fromString;
					    toRegister = toString;
					} // 0 < to < from < 4092
				    } catch (NumberFormatException nfe) {
					// System.out.println("no");
					popupError("To must be an integer");
				    } // is to an integer
				} // is to a combobox
			    } // has to changed?
			} catch (NumberFormatException nfe) {
			    // System.out.println("no");
			    popupError("From must be an integer");
			} // is from an integer
		    } // is from a combobox
		} // has from changed?
		
		updateData();

	    } // actionPerformed
	});
*/

/*

    private int from() {
	int from, to;
	String tempFrom = fromRegister;
	try {
	    from = (new Integer(fromRegister)).intValue();
	    // from is an integer
	    while (from%4 != 0)
		from--;
	    return from/4;
	} catch (NumberFormatException nfe) {
	    // from is a register. get its value
	    try {
		from =  getMemoryValueAtRegister(fromRegister);
		if (from == 4096)
		    from = 4092;
		try {
		    to = (new Integer(toRegister)).intValue();
		    // to is an integer
		    if (from < to || from > 4092) {
			// from is lower than to
			fromRegister = "4092";
			// System.out.println(fromRegister +"=" + from);
			// System.out.println(toRegister +"="+ to);
			popupError(tempFrom + " got out of the bounds: [" + toRegister + ", 4092]");
			return 1024;
		    } else {
			// from > to
			return from/4;
		    } // is from lower than to
		} catch (NumberFormatException nfe1) {
		    // to is a register
		    if (0 > from || from > 4092) {
			// from got out of bounds
			fromRegister = "4092";
			// System.out.println(fromRegister +"=" + from);
			popupError(tempFrom + " got is out of the bounds: [0, 4092]");
			return 1024;
		    } else {
			// from is in the bounds
			return from/4;
		    } // is from inside the bounds?
		} // is to an integer?
	    } catch (DataFormatException dfe) {
		fromRegister = "4092";
		popupError(fromRegister + " is not a known format");
		return 1024;
	    }
	} // is from an integer?
    } // from()

    private int to() {
	int to;
	String tempTo = toRegister;
	
	try {
	    to = (new Integer(toRegister)).intValue();
	    // to is an integer
	    while (to%4 != 0)
		to--;
	    return to/4;
	} catch (NumberFormatException nfe) {
	    // to is a register
	    try {
		to = getMemoryValueAtRegister(toRegister);
		// if to is 4096, set it to 4092:
		if (to == 4096)
		    to = 4092;
		if (to > from()*4 || to < 0) {
		    // to is higher than from
		    toRegister = "0";
		    popupError(tempTo + " got out of the bounds [0, " + fromRegister + "]");
// 		    System.out.println("to=" + to);
// 		    System.out.println("from=" + from());
		    return 0;
		} else {
		    // to is inside the bounds
		    return to/4;
		} // is to inside the bounds?
	    } catch (DataFormatException dfe) {
		toRegister = "0";
		popupError(tempTo + "is not a known format");
		return 0;
	    }
	} //is to an integer?
    } // to()
*/

/*

    protected void updateData() {
	
	/** filling the stackTable * /
	stackTableSize = (from() - to())+1;
	stackTable = new Object[stackTableSize][2];
	int j = 0;
	int k;
	String hexaString, tempString;
	for (int i=from(); i >= to(); i--) {
	    // Now take care, 'cause the address should be in inversed order
	    stackTable[j][0] = new Integer(i*4);
	    if (hexa) {
		hexaString = Integer.toHexString(startGUI.getEmulator().getWordAt(i*4));
		tempString = "";
		for(k=0;k<hexaString.length();k++) {
		    tempString =  tempString + hexaString.substring(k, k+1);
		}
		for(; k<8; k++) {
		    tempString = "0" + tempString;
		}
		//	System.out.println(";"+tempString+";");
		stackTable[j][1] = tempString;
	    } else {
		stackTable[j][1] = new Integer(startGUI.getEmulator().getWordAt(i*4));	
	    }
	    j++;
	}

	/** Setting the preferred size for the columns * /
	TableColumn column = null;
	column = stackJTable.getColumnModel().getColumn(0);
	column.setPreferredWidth(40);
	column.setMaxWidth(60);

	/** filling the stackRiscTable * /
	stackRiscTable.update(columnNames, stackTable);
	
	/** Displays the bounds limits * /
	if (fromRegister.charAt(0) == 'R' ||
	    fromRegister.equalsIgnoreCase("sp") ||
	    fromRegister.equalsIgnoreCase("fp") ||
	    fromRegister.equalsIgnoreCase("lnk") ||
	    fromRegister.equalsIgnoreCase("4092")){
	    fromComboBox.setSelectedItem(fromRegister);
	    fromTextField.setText("");
	} else {
	    fromComboBox.setSelectedItem("<-");
	    fromTextField.setText(fromRegister);
	}
	if (toRegister.charAt(0) == 'R' ||
	    toRegister.equalsIgnoreCase("sp") ||
	    toRegister.equalsIgnoreCase("fp") ||
	    toRegister.equalsIgnoreCase("lnk") ||
	    toRegister.equalsIgnoreCase("4092")){
	    toComboBox.setSelectedItem(toRegister);
	    toTextField.setText("");
	} else {
	    toComboBox.setSelectedItem("<-");
	    toTextField.setText(toRegister);
	}
	
    }
*/

/*

    public int getMemoryValueAtRegister(String reg) throws DataFormatException {
	if (reg.equalsIgnoreCase("sp"))
	    return startGUI.getEmulator().getRx(startGUI.SP);
	if (reg.equalsIgnoreCase("fp"))
	    return startGUI.getEmulator().getRx(startGUI.FP);
	if (reg.equalsIgnoreCase("lnk"))
	    return startGUI.getEmulator().getRx(startGUI.LNK);
	if (reg.charAt(0)=='R' || reg.charAt(0)=='r')
	    return startGUI.getEmulator().getRx((new Integer(reg.substring(1))).intValue());
	throw new DataFormatException("Bad Register name format");
    }
*/

/*

    private void popupError(String errorMessage) {
	JOptionPane.showMessageDialog(null,
				      errorMessage,
				      "Error",
				      JOptionPane.ERROR_MESSAGE);
	// System.out.println("an error occured");
	updateData();
    } // popupError
*/
