package risc.syscall;

import java.util.Arrays;

import risc.RISC;
import risc.emulator.Emulator;

public class Dispatcher extends AbstractHandler implements Dispatchee {

    //########################################################################
    // Private state

    private final int[] syscalls;
    private final Dispatchee[] handlers;

    //########################################################################
    // Constructors

    public Dispatcher(Dispatchee[] dispatchees) {
        int[][] sctable = new int[dispatchees.length][];
        int cnt = 0;
        int min = 0;
        int max = 0;
        for (int i = 0; i < dispatchees.length; i++) {
            sctable[i] = dispatchees[i].getHandledSysCalls();
            cnt = cnt + sctable[i].length;
            for (int j = 0; j < sctable[i].length; j++) {
                min = Math.min(min, sctable[i][j]);
                max = Math.max(max, sctable[i][j]);
            }
        }
        if (min == 0 && max <= 2 * cnt) {
            this.syscalls = null;
            this.handlers = new Dispatchee[max + 1];
            for (int i = 0; i < sctable.length; i++) {
                for (int j = 0; j < sctable[i].length; j++) {
                    int k = sctable[i][j];
                    if (handlers[k] != null)
                        throw new Error("multiple occurrences of " + k);
                    handlers[k] = dispatchees[i];
                }
            }
        } else {
            this.syscalls = new int[cnt];
            this.handlers = new Dispatchee[cnt];
            for (int i = 0; i < sctable.length; i++) {
                for (int j = 0; j < sctable[i].length; j++) {
                    syscalls[--cnt] = sctable[i][j];
                }
            }
            Arrays.sort(syscalls);
            for (int i = 0; i < sctable.length; i++) {
                for (int j = 0; j < sctable[i].length; j++) {
                    int k = Arrays.binarySearch(syscalls, sctable[i][j]);
                    if (handlers[k] != null)
                        throw new Error("multiple occurrences of " + k);
                    handlers[k] = dispatchees[i];
                }
            }
        }
    }

    //########################################################################
    // Dispatcher interface

    public Dispatchee lookup(int sc) {
        if (syscalls == null) {
            return (0 <= sc && sc < handlers.length) ? handlers[sc] : null;
        } else {
            int k = Arrays.binarySearch(syscalls, sc);
            return k >= 0 ? handlers[k] : null;
        }
    }

    //########################################################################
    // Dispatchee interface

    public int[] getHandledSysCalls() {
        int[] src;
        int cnt;
        if (syscalls == null) {
            src = new int[handlers.length];
            cnt = 0;
            for (int i = 0; i < handlers.length; i++) {
                if (handlers[i] != null) src[cnt++] = i;
            }
        } else {
            src = syscalls;
            cnt = syscalls.length;
        }
        int[] dst = new int[cnt];
        System.arraycopy(src, 0, dst, 0, cnt);
        return dst;
    }

    //########################################################################
    // SysCallHandler interface

    public int handle(int sc, int a, int b, Emulator emu)throws Emulator.Exception{
        Dispatchee handler = lookup(sc);
        if (handler != null) return handler.handle(sc, a, b, emu);
        return super.handle(sc, a, b, emu);
    }

    //########################################################################
}
